#!/usr/bin/python3
# -*- coding: utf-8 -*-

##imports
#stdlib
import os
import getpass
import urllib.error
#self
import lib.db
import lib.wizard
from lib.tools import print_installed_dict, ask_yes
from lib.text import *
from lib.wizard import PackageWizard
from core import __version__, __version_name__, __poetry__
#other


__all__ = ['list', 'upgrade','update', 'freeze', 'install', 'roll', 'uninstall', 'info', 'flush']

##cmd
#info
def info(options):
    if options['release']:
        if options['whereis']:
            print(bold(blue('Version: ')), end='')
        print(bold(green(__version__+'/'+__version_name__)))
    if options['whereis']:
        print(bold(blue('Whereis:')))
        print(bold(blue(' core.py:  '))+green(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'core.py')))
        if os.path.isfile(os.path.join(options['config'])):
            print(bold(blue(' conf:     '))+green(options['config']))
        else:
            print(bold(blue(' conf:     '))+yellow('None (run git-pm as superuser to create one)'))
        if os.path.isfile(os.path.join(os.path.dirname(options['config']), 'database.json')):
            print(bold(blue(' db:       '))+green(os.path.join(os.path.dirname(options['config']), 'database.json')))
        else:
            print(bold(blue(' db:       '))+yellow('None (run git-pm as superuser to create one)'))
        print(bold(blue(' log:      '))+green(options['log']['path']))
        print(bold(blue(' temp_dir: '))+green(options['dir']['temp']))
        print(bold(blue(' inst_dir: '))+green(options['dir']['inst']))

    if options['poetry']:
        print(bold(grey(__version_name__)))
        print(invert(bold(center(__poetry__, True))))

def flush(options):
    print(yellow('this will remove all files installed by git-pm'))
    if ask_yes('are you sure you wish to do this ?', False):
        #db file
        print(bold(blue('Starting flush: ')))
        db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'), True)
        for package in db.getAllRepos():
            if not package == 'althonos/git-pm':
                print(blue('  uninstalling {}'.format(package)))
                wizard = PackageWizard(package, options, db.getRepoDetails(package))
                wizard.uninstall()
                db.deregister(package)
                db.dump()
        #config file
        try:
            os.remove(options['config'])
            print(blue('  deleting config file ({})'.format(options['config'])))
        except FileNotFoundError:
            print(yellow('  no config file to delete'))
        #temp dir
        try:
            shutil.rmtree(options['dir']['temp'])
            print(blue('  deleting temp directory ({})'.format(options['dir']['temp'])))
        except FileNotFoundError:
            print(yellow('  no temp directory to delete'))
        #log dir
        try:
            os.remove(options['log']['path'])
            print(blue('  deleting log file ({})'.format(options['log']['path'])))
        except:
            print(yellow('  no log file to delete'))
        #installdir
        try:
            shutil.rmtree(options['dir']['inst'])
            print(blue('  deleting install directory ({})'.format(options['dir']['inst'])))
        except FileNotFoundError:
            print(yellow('  no install directory to delete'))

        print(bold(green('Flush was successful')))



#list
def list(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    repo_list = db.getFilteredRepos(options)
    if options['pipe']:
        print(' '.join(repo_list))
    else:
        if not repo_list:
            print(yellow('No packages found.'))
        else:
            repo_list_detailled = {repo: db.getRepoDetails(repo) for repo in repo_list}
            print_installed_dict(repo_list_detailled, infos=options['infos'])

#update
def update(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    repo_list = db.getAllRepos()
    if not repo_list:
        print(yellow('No packages found.'))
    else:
        for index, repo in enumerate(repo_list):
            try:
                print(blue('Checking commits of {}'.format(db.getRepo(repo))))
                uptodate = db.update(repo)
            except urllib.error.URLError:
                print('')
                options['parser']['update'].error('impossible to connect to internet')
        db.dump()

    uptodate_list = db.getReposUpToDate(True)
    toupgrade_list = [x for x in db.getReposUpToDate(False) if x not in db.getReposByState('frozen')]
    wontupgrade_list = [x for x in db.getReposUpToDate(False) if x in db.getReposByState('frozen')]

    if uptodate_list:
        print(green('{} package{} up-to-date:\n '.format(len(uptodate_list), [' is', 's are'][len(uptodate_list) > 1])), end='')
        print(green(' '.join(uptodate_list)))
    if toupgrade_list:
        print(yellow('{n} package{s} an upgrade:\n '.format(n = len(toupgrade_list), s = [' needs', 's need'][len(toupgrade_list) > 1])), end='')
        print(yellow(' '.join(toupgrade_list)))
    if wontupgrade_list:
        print(cyan('{} package{} will be ignored:\n '.format(len(wontupgrade_list), ['', 's'][len(wontupgrade_list) > 1])), end='')
        print(cyan(' '.join(wontupgrade_list)))


#freeze
def freeze(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    freeze_list = db.getFilteredRepos(options)
    frozen_list = db.getReposByState('frozen')
    new_list, already_list = [], []
    new, already = 0, 0
    if not freeze_list:
        print(yellow('No packages found.'))
    else:
        for package in freeze_list:
            if package not in db.getAllRepos() and package not in db.getAllNames():
                options['parser']['freeze'].error('package {} was not found in database'.format(package))
        for package in freeze_list:
            if db.getRepo(package) in frozen_list:
                print(bold(yellow('Package {} is already frozen'.format(db.getRepo(package)))))
                already += 1
                already_list.append(db.getRepo(package))
            else:
                print(bold(blue('Freezing {}'.format(db.getRepo(package)))))
                db.freeze(package)
                db.dump()
                new += 1
                new_list.append(db.getRepo(package))
        if new:
            print(bold(green('{} new package{} frozen:\n'.format(new, [' was', 's were'][new>1]))),
                  ' ', bold(green(' '.join(new_list))))
        if already:
            print(bold(yellow('{} package{} already frozen:\n'.format(already, [' was', 's were'][already>1]))),
                  ' ', bold(yellow(' '.join(already_list))))


#roll
def roll(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    roll_list = db.getFilteredRepos(options)
    rolling_list = db.getReposByState('rolling')
    new_list, already_list = [], []
    new, already = 0, 0
    if not roll_list:
        print(yellow('No packages found.'))
    else:
        for package in roll_list:
            if package not in db.getAllRepos() and package not in db.getAllNames():
                options['parser']['roll'].error('package {} was not found in database'.format(package))
        for package in roll_list:
            if db.getRepo(package) in rolling_list:
                print(bold(yellow('Package {} is already rolling'.format(db.getRepo(package)))))
                already += 1
                already_list.append(db.getRepo(package))
            else:
                print(bold(blue('Rolling {}'.format(db.getRepo(package)))))
                db.roll(package)
                db.dump()
                new += 1
                new_list.append(db.getRepo(package))
        if new:
            print(bold(green('{} new package{} rolled:\n'.format(new, [' was', 's were'][new>1]))),
                  ' ', bold(green(' '.join(new_list))))
        if already:
            print(bold(yellow('{} package{} already rolling:\n'.format(already, [' was', 's were'][already>1]))),
                  ' ', bold(yellow(' '.join(already_list))))


#upgrade
def upgrade(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    if options['packages'] is not None:
        for package in pbar(options['packages']):
            if package not in db:
                options['parser']['upgrade'].error('package {} was not found in database'.format(package))

    upgrade_list = db.getFilteredRepos(options)
    if not upgrade_list:
        print(yellow('No packages found.'))
    else:
        upgrade_list = [x for x in db.getReposUpToDate(False) if x in upgrade_list]
        toupgrade_list = [x for x in upgrade_list if x not in db.getReposByState('frozen')]
        wontupgrade_list = [x for x in upgrade_list if x in db.getReposByState('frozen')]

        if toupgrade_list:
            print(yellow('{} package{} will be upgraded:\n '.format(len(toupgrade_list), ['', 's'][len(toupgrade_list) > 1])), end='')
            print(yellow(' '.join(toupgrade_list)))
        else:
            print(green('All packages are up-to-date.'))
            return


        for package in toupgrade_list:
            print(bold(blue('Upgrading {}:'.format(db.getRepo(package)))))
            wizard = PackageWizard(package, options, db.getRepoDetails(package))

            try:
                wizard.uninstall()
            except FileNotFoundError:
                pass

            wizard.clone()
            wizard.install(options['verbose'])
            print(blue('  registering {} in database'.format(wizard.git_name)))
            db.register(wizard, options)
            db.dump()
            print(bold(green('  package {} was upgraded successfully (commit {})'.format(wizard.git_name, db.getRepoDetails(wizard.git_name)['installed']))))


#install
def install(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    new, already = 0,0
    new_list, already_list = [], []
    for package in options['packages']:
        if package in db and len(options['packages']) == 1:
            commit = db.getRepoDetails(db.getRepo(package))['installed']
            options['parser']['install'].error("package {} is already installed (commit {}), use upgrade to install a new version if available".format(db.getRepo(package), commit))
        elif package in db:
            commit = db.getRepoDetails(db.getRepo(package))['installed']
            print(bold(yellow('Package {} is already installed (commit {})'.format(package, commit))))
            already += 1
            already_list.append(db.getRepo(package))
        else:
            print(bold(blue('Installing {}:'.format(package))))
            try:
                wizard = PackageWizard(package, options)
            except urllib.error.URLError:
                options['parser']['install'].error('impossible to connect to internet')
            except AssertionError:
                options['parser']['install'].error('impossible to resolve {}'.format(package))
            try:
                wizard.clone()
            except AssertionError:
                options['parser']['install'].error('impossible to clone {}'.format(package))
            try:
                wizard.install(verbose=options['verbose'])
            except AssertionError:
                options['parser']['install'].error('could not install {} with {} installation'.format(package, wizard.install_type))

            print(blue('  registering {} in database'.format(wizard.git_name)))
            db.register(wizard, options)
            db.dump()
            print(bold(green('  package {} was installed successfully (commit {})'.format(wizard.git_name, db.getRepoDetails(wizard.git_name)['installed']))))
            new += 1
            new_list.append(wizard.git_name)
    if new:
        print(bold(green('{} new package{} installed:\n'.format(new, [' was', 's were'][new>1]))),
              ' ', bold(green(' '.join(new_list))))
    if already:
        print(bold(yellow('{} package{} already installed:\n'.format(already, [' was', 's were'][already>1]))),
              ' ', bold(yellow(' '.join(already_list))))

#uninstall
def uninstall(options):
    db = lib.db.DB(os.path.join(os.path.dirname(options['config']), 'database.json'))
    new, new_list = 0, []
    package_list = db.getFilteredRepos(options)

    for package in package_list:
        if package not in db:
            options['parser']['uninstall'].error("package {} was not found in database".format(package))
        else:
            if db.getRepo(package) == "althonos/git-pm":
                if len(db.getAllRepos()) > 1:
                    options['parser']['uninstall'].error("you should clean your system with git-pm flush before uninstalling git-pm".format(package))
            package = db.getRepo(package)
            print(bold(blue('Uninstalling {}:'.format(package))))
            if db.getRepo(package) == "althonos/git-pm": print(invert(fill('  Goodbye, cruel world !')))
            wizard = PackageWizard(package, options, db.getRepoDetails(package))
            wizard.uninstall(verbose=options['verbose'])
            print(blue('  deregistering {} from database'.format(package)))
            db.deregister(package)
            db.dump()
            new += 1
            new_list.append(package)
            print(bold(green('  package {} was uninstalled successfully'.format(package))))
    if new:
        print(bold(green('{} package{} uninstalled:\n'.format(new, [' was', 's were'][new>1]))),
              ' ', bold(green(' '.join(new_list))))
