#!/usr/bin/python3
# -*- coding: utf-8 -*-

##imports
#stdlib
import collections
import shutil
import sys
import os
from datetime import datetime
import urllib.request as rqst
#self
from lib.text import *
#other
from bs4 import BeautifulSoup


##functions
#transtyping
def deep_convert_dict(layer):
    """
    http://stackoverflow.com/questions/15711755/converting-dict-to-ordereddict
    """
    to_ret = layer
    if isinstance(layer, collections.OrderedDict):
        to_ret = dict(layer)
    try:
        for key, value in to_ret.items():
            to_ret[key] = deep_convert_dict(value)
    except AttributeError:
        pass
    return to_ret

#web
def get_soup(url):
    req = rqst.Request(url, headers={'User-Agent':'firefox'})
    con = rqst.urlopen(req)
    soup = BeautifulSoup(con.read(), 'html.parser')
    return soup

def touch(url):
    req = rqst.Request(url, headers={'User-Agent':'firefox'})
    con = rqst.urlopen(req)

#io
def ask_yes(message, default_answer=False):
    default = ['[y/N]', '[Y/n]']
    answer = input(yellow(message.strip() + ' ' + default[default_answer]))
    if not answer:
        return default_answer
    else:
        if answer[0].upper() in ('Y', 'N'):
            return answer[0].upper() == 'Y' #true if answer=Y, false if answer=N
        else:
            print(red('unrecognized answer: {}'.format(answer)))
            return ask_yes(message, default_answer)

#printer
def print_installed_dict(installed_dict, infos=['name', 'author', 'installed', 'latest', 'state', 'url', 'install_type']):
    max_len = {x: len(x) for x in infos}
    #checks len of dict
    for repo in installed_dict.keys(): #checks len of dict entries
        for key in infos:
            if len(installed_dict[repo][key]) > max_len[key]:
                max_len[key] = len(installed_dict[repo][key])

    abs_len = sum([max_len[key] for key in infos])+len(infos)
    if abs_len > shutil.get_terminal_size()[0]:
        print_installed_dict(installed_dict, infos=infos[:-1])
        return
    #prints the header
    for key in infos:
        print(grey(key.ljust(max_len[key])), end=' ')
    print()
    #prints '=' frame
    for key in infos: #print the frame
        print(grey('='*max_len[key]), end=grey('='))
    print()
    #prints packages
    for repo in installed_dict.keys(): #print the packages infos with colorscheme
        for key in infos:
            if installed_dict[repo]['state'] == 'frozen':
                print(cyan(installed_dict[repo][key].ljust(max_len[key])), end=' ')
            else:
                if installed_dict[repo]['installed'] == installed_dict[repo]['latest']:
                    print(green(installed_dict[repo][key].ljust(max_len[key])), end=' ')
                else:
                    print(yellow(installed_dict[repo][key].ljust(max_len[key])), end=' ')
        print()

#log
class Logger(object):
    """ http://stackoverflow.com/questions/4675728"""
    def __init__(self, filepath = "./git-pm.log", err=False, quiet=False):

        self.err = err
        self.quiet = quiet
        if not err:
            self.terminal = sys.stdout
        else:
            self.terminal = sys.stderr

        self.log = open(filepath, 'a+')
        os.chmod(filepath, 0o664)
        if not self.err:
            self.log.write('-\n')
            self.log.write('[{}] '.format(datetime.now())+' '.join(sys.argv)+'\n')

    def write(self, message):
        if not self.quiet: self.terminal.write(message)
        if message.strip():
            message = '[{}] '.format(datetime.now()) + message.strip()
            if not '\n' in message: message += '\n'
            self.log.write(message)

    def flush(self):
        self.log.flush()
