#!/usr/bin/python3
# -*- coding: utf-8 -*-

__version__ = '1.0'
__version_name__ = 'Apollinaire'

##imports
#stdlib
import subprocess
import os
import shutil
#self
from lib.tools import get_soup, touch
from lib.text import *

##class
class PackageWizard:


    ##init and overcharge
    def __init__(self, package_id, options, db_entry=None):

        if db_entry is None:
            self.__setUrl(package_id)
            assert self.url is not None
        else:
            self.url = db_entry['url']

        self.name = self.url.split('/')[-1]
        self.author = self.url.split('/')[-2]
        self.git_name = self.author+'/'+self.name
        self.absolute_name = self.author+'.'+self.name
        self.tmp_path = os.path.join(options['dir']['temp'], self.absolute_name)
        self.standalone_path = os.path.join(options['dir']['inst'], self.absolute_name)

        if db_entry is not None:
            self.install_type = db_entry['install_type']
            self.installed_files = db_entry['installed_files']
            self.ALWAYS_STANDALONE = False
        else:
            self.ALWAYS_STANDALONE = options['standalone']

    def __setLatestCommit(self):
        soup = get_soup(self.url)
        self.latest_commit = soup.find('a', {'class':'commit-tease-sha'}).get_text().split()[0]

    def __setUrl(self, package_id):
        """
        Looks for the right url of the package
        """
        if len(package_id.split('/'))==2: #author/repo
            web_path = package_id
            self.url = "http://github.com/" + web_path
        elif len(package_id.split('/'))==1: #repo
            print(blue("  searching for {}".format(package_id)))
            search_soup = get_soup("http://github.com/search?q={query}".format(query=package_id))
            package_sections = search_soup.find_all('h3')
            if "We’ve" in package_sections[1].get_text():
                package_section = package_sections[2]
            else:
                package_section = package_sections[1]
            web_path = package_section.get_text().split()[0]
            print(blue("  assuming {} is {}".format(package_id, web_path)))
            self.url = "http://github.com/" + web_path
        elif 'http' in package_id:
            print(blue("  checking url {}".format(package_id)))
            self.url = package_id
            try:
                touch(self.url)
            except (urllib.errors.URLError, urllib.errors.HTTPError):
                self.url = None
        else:
            self.url = None

    def __setInstallType(self):
        content = os.listdir(self.tmp_path)
        if self.ALWAYS_STANDALONE:
            self.install_type = 'standalone'
        else:
            if 'setup.py' in content:
                self.install_type = 'setup.py'
            elif 'configure' in content:
                self.install_type = 'configure'
            else:
                self.install_type = 'standalone'

    ##attr
    def clone(self):
        if not os.path.isdir(os.path.dirname(self.tmp_path)):
            os.mkdir(os.path.dirname(self.tmp_path))
        if os.path.isdir(self.tmp_path):
            shutil.rmtree(self.tmp_path)
        print(blue('  cloning {} to {}'.format(self.git_name, self.tmp_path)))
        out, err =  subprocess.Popen(['git', 'clone', self.url, self.tmp_path, '--progress'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT).communicate()

        assert os.path.isdir(self.tmp_path)
        self.__setInstallType()

    def install(self, verbose=False):
        cmd_path = os.getcwd()
        self.__setInstallType()
        self.__setLatestCommit()
        if self.install_type == 'standalone':
            self.installed_files = [self.standalone_path]
            if not os.path.isdir(os.path.dirname(self.standalone_path)):
                os.mkdir(os.path.dirname(self.standalone_path))
                os.chmod(os.path.dirname(self.standalone_path), 0o775)
            print(blue('  moving {} to {}'.format(self.git_name, self.standalone_path)))
            shutil.move(self.tmp_path, self.standalone_path)

        if self.install_type == 'setup.py':
            print(blue('  runnning {}'.format(os.path.join(self.tmp_path, 'setup.py'))))
            os.chdir(self.tmp_path)
            p = subprocess.Popen(['python3', 'setup.py', 'install',
                                  '--record', 'records.txt'],
                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = p.communicate()

            for line in out.decode('utf-8').split('\n'):
                if line and verbose:
                    print(blue('    '+line.strip()))

            self.installed_files = open('records.txt', 'r').read().split('\n')
            assert self.installed_files
            os.chdir(cmd_path)

    def uninstall(self, verbose=False):
        for installed_file in self.installed_files:
            if installed_file:
                if verbose: print(blue('  deleting {}'.format(installed_file)))
                if os.path.isdir(installed_file):
                    shutil.rmtree(installed_file)
                elif os.path.isfile(installed_file):
                    os.remove(installed_file)
